from passbyme2fa_client import *
import unittest
import os
import httpretty
from pprint import pprint
import arrow

class TestCancelMessage(unittest.TestCase):

    def setUp(self):
        pem_file = os.path.join(os.path.dirname(__file__), "auth.pem")
        self.pbm = PassByME2FAClient(
            key = pem_file, cert = pem_file, password = "123456"
        )
        httpretty.enable()

    def test_response_body(self):
        httpretty.register_uri(httpretty.DELETE, "https://auth-sp.passbyme.com/frontend/messages/YzX95zUA1et2ijQ",
           status = 200,
           body = "{\"messageId\": \"YzX95zUA1et2ijQ\", \"expirationDate\" : \"2015-06-11T13:06:12.658+02:00\"," +
                     "\"recipients\" : [{ \"userId\": \"pbmId1\", \"status\": \"CANCELLED\" }]}")
        session_info = self.pbm.cancel_message(
                "YzX95zUA1et2ijQ"
            )
        self.assertEqual("YzX95zUA1et2ijQ", session_info.message_id)
        self.assertEqual(arrow.get("2015-06-11T13:06:12.658+02:00"), session_info.expiration_date)
        self.assertEqual(1, len(session_info.recipient_statuses))
        self.assertEqual("pbmId1", session_info.recipient_statuses[0].user_id)
        self.assertEqual(PassByME2FAClient.MessageStatus.CANCELLED, session_info.recipient_statuses[0].status)

    def test_session_info_cancel(self):
        httpretty.register_uri(httpretty.POST, "https://auth-sp.passbyme.com/frontend/messages",
           status = 200,
           body = "{\"messageId\": \"YzX95zUA1et2ijQ\", \"expirationDate\" : \"2015-06-11T13:06:12.658+02:00\"," +
                     "\"recipients\" : [{ \"userId\": \"pbmId1\", \"status\": \"PENDING\" }]}")
        httpretty.register_uri(httpretty.GET, "https://auth-sp.passbyme.com/frontend/messages/YzX95zUA1et2ijQ",
           status = 200,
           body = "{\"messageId\": \"YzX95zUA1et2ijQ\", \"expirationDate\" : \"2015-06-11T13:06:12.658+02:00\"," +
                     "\"recipients\" : [{ \"userId\": \"pbmId1\", \"status\": \"CANCELLED\" }]}")

        session_info = self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = 300,
                type = PassByME2FAClient.MessageType.MESSAGE
            )
        self.assertEqual("YzX95zUA1et2ijQ", session_info.message_id)
        self.assertEqual(arrow.get("2015-06-11T13:06:12.658+02:00"), session_info.expiration_date)
        self.assertEqual(1, len(session_info.recipient_statuses))
        self.assertEqual("pbmId1", session_info.recipient_statuses[0].user_id)
        self.assertEqual(PassByME2FAClient.MessageStatus.PENDING, session_info.recipient_statuses[0].status)

        session_info.cancel()

        self.assertEqual(PassByME2FAClient.MessageStatus.CANCELLED, session_info.recipient_statuses[0].status)



from .json_helper import get_json_field

class RecipientStatus:

    def __init__(self, json):
        self.user_id = get_json_field(json, "userId")
        self.status = get_json_field(json, "status")        




try:
    from httplib import HTTPSConnection
except:
    from http.client import HTTPSConnection
import ssl
import os
from pprint import pprint
import json
from .http_error import HTTPError
from .passbyme_error import PassByMEError
from .session_info import SessionInfo

class PassByME2FAClient:

    class MessageType:
        AUTHORIZATION = "authorization"
        MESSAGE       = "message"
        ESIGN         = "esign"

    class MessageStatus:
        PENDING        = "PENDING"
        NOTIFIED       = "NOTIFIED"
        DOWNLOADED     = "DOWNLOADED"
        SEEN           = "SEEN"
        NOT_SEEN       = "NOT_SEEN"
        NOT_NOTIFIED   = "NOT_NOTIFIED"
        NOT_DOWNLOADED = "NOT_DOWNLOADED"
        NO_DEVICE      = "NO_DEVICE"
        FAILED         = "FAILED"
        DISABLED       = "DISABLED"
        CANCELLED      = "CANCELLED"
        APPROVED       = "APPROVED"
        DENIED         = "DENIED"

    def __init__(self,
                address = "auth-sp.passbyme.com",
                key = None, cert = None, password = None
            ):
        if key is None or cert is None or len(key) == 0 or len(cert) == 0:
            raise ValueError("SSL key or certificate is missing!")
        context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
        context.options |= ssl.OP_NO_SSLv2
        context.options |= ssl.OP_NO_SSLv3
        context.verify_mode = ssl.CERT_REQUIRED
        context.check_hostname = True
        context.load_verify_locations(
            os.path.join(os.path.dirname(__file__), "truststore.pem"))
        context.load_cert_chain(certfile = cert,
            keyfile = key, password = password)
        self.http = HTTPSConnection(address,
                context = context
            )

    def send_message(self,
                recipients = None,
                subject = None,
                body = None,
                availability = None,
                type = None,
                callbackUrl = None):
        if recipients is None or len(recipients) == 0:
            raise ValueError("Recipients are not given!")

        availability = int(availability)
        if availability < 1:
            raise ValueError("Invalid availability!")

        if type not in [getattr(self.MessageType, attr)
                for attr in dir(self.MessageType) if not attr.startswith("__")]:
            raise ValueError("Invalid type: " + type)

        body = json.dumps({
          "recipients": recipients,
          "subject": subject,
          "body": body,
          "availability": availability,
          "type": type,
          "callbackUrl": callbackUrl
        })
        headers = {
            "Content-type": "application/json",
        }
        return SessionInfo(
            self.__do_https("POST", "/frontend/messages", headers, body),
            self
        )

    def track_message(self, message_id = None, session_info = None):
        return self.__handle_existing_message("GET",
            message_id, session_info)

    def cancel_message(self, message_id = None, session_info = None):
        return self.__handle_existing_message("DELETE",
            message_id, session_info)

    def __handle_existing_message(self, http_method,
                message_id = None, session_info = None
            ):
        if session_info is not None:
            message_id = session_info.message_id
        if message_id is None or len(message_id) == 0:
            raise ValueError("Message id is missing!")

        json = self.__do_https(http_method, "/frontend/messages/" + message_id)

        if session_info is not None:
            session_info.re_init(json)
            return session_info

        return SessionInfo(json, self)

    def __do_https(self, method, path, additional_headers = None, body = None):
        headers = {
            "X-PBM-API-VERSION": "1"
        }
        if additional_headers is not None:
            headers.update(additional_headers)

        self.http.request(method, path, body, headers)
        response = self.http.getresponse()

        resp_body = response.read()
        if len(resp_body) == 0:
            raise HTTPError(response)

        if response.status < 200 or response.status > 299:
            if response.status == 420:
                raise PassByMEError(json.loads(resp_body.decode("utf-8")))
            else:
                raise HTTPError(response)

        return json.loads(resp_body.decode("utf-8"))



from passbyme2fa_client import *
import unittest

class TestConfig(unittest.TestCase):

    def test_should_miss_key(self):
        with self.assertRaises(ValueError):
            PassByME2FAClient()

    def test_should_miss_cert(self):
        with self.assertRaises(ValueError):
            PassByME2FAClient(key = "Given")



from passbyme2fa_client import *
import unittest
import os
import httpretty
from pprint import pprint
import arrow

class TestSendMessage(unittest.TestCase):

    def setUp(self):
        pem_file = os.path.join(os.path.dirname(__file__), "auth.pem")
        self.pbm = PassByME2FAClient(
            key = pem_file, cert = pem_file, password = "123456"
        )
        httpretty.enable()

    def test_missing_recipients(self):
        with self.assertRaises(ValueError):
            self.pbm.send_message(
            )
        with self.assertRaises(ValueError):
            self.pbm.send_message(
                recipients = []
            )

    def test_invalid_availability(self):
        with self.assertRaises(ValueError):
            self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = "hey"
            )

    def test_invalid_type(self):
        with self.assertRaises(ValueError):
            self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = 300,
                type = "whoa"
            )

    def test_non_420_error_code(self):
        httpretty.register_uri(httpretty.POST, "https://auth-sp.passbyme.com/frontend/messages",
                           status = 500)
        with self.assertRaises(HTTPError):
            self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = 300,
                type = PassByME2FAClient.MessageType.MESSAGE
            )

    def test_420_error_code_no_resp_body(self):
        httpretty.register_uri(httpretty.POST, "https://auth-sp.passbyme.com/frontend/messages",
                           status = 420, body = None)
        with self.assertRaises(HTTPError):
            self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = 300,
                type = PassByME2FAClient.MessageType.MESSAGE
            )

    def test_420_error_code_invalid_resp_body(self):
        httpretty.register_uri(httpretty.POST, "https://auth-sp.passbyme.com/frontend/messages",
                           status = 420, body = "No JSON, sorry.")
        with self.assertRaises(ValueError):
            self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = 300,
                type = PassByME2FAClient.MessageType.MESSAGE
            )

    def test_420_error_code(self):
        httpretty.register_uri(httpretty.POST, "https://auth-sp.passbyme.com/frontend/messages",
                           status = 420,
                           body = "{\"code\": 421}")
        with self.assertRaises(PassByMEError) as context:
            self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = 300,
                type = PassByME2FAClient.MessageType.MESSAGE
            )
        self.assertEqual(421, context.exception.response["code"])

    def test_no_response_body(self):
        httpretty.register_uri(httpretty.POST, "https://auth-sp.passbyme.com/frontend/messages",
                           status = 200, body = None)
        with self.assertRaises(HTTPError):
            self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = 300,
                type = PassByME2FAClient.MessageType.MESSAGE
            )

    def test_response_body(self):
        httpretty.register_uri(httpretty.POST, "https://auth-sp.passbyme.com/frontend/messages",
           status = 200,
           body = "{\"messageId\": \"YzX95zUA1et2ijQ\", \"expirationDate\" : \"2015-06-11T13:06:12.658+02:00\"," +
                     "\"recipients\" : [{ \"userId\": \"pbmId1\", \"status\": \"PENDING\" }," +
                       "{\"userId\": \"pbmId2\", \"status\": \"NOTIFIED\" },"+
                       "{\"userId\": \"pbmId3\", \"status\": \"SEEN\" }]}")
        session_info = self.pbm.send_message(
                recipients = ["somebody@somewhe.re"],
                availability = 300,
                type = PassByME2FAClient.MessageType.MESSAGE
            )
        self.assertEqual("YzX95zUA1et2ijQ", session_info.message_id)
        self.assertEqual(arrow.get("2015-06-11T13:06:12.658+02:00"), session_info.expiration_date)
        self.assertEqual(3, len(session_info.recipient_statuses))
        self.assertEqual("pbmId1", session_info.recipient_statuses[0].user_id)
        self.assertEqual(PassByME2FAClient.MessageStatus.PENDING, session_info.recipient_statuses[0].status)
        self.assertEqual("pbmId2", session_info.recipient_statuses[1].user_id)
        self.assertEqual(PassByME2FAClient.MessageStatus.NOTIFIED, session_info.recipient_statuses[1].status)
        self.assertEqual("pbmId3", session_info.recipient_statuses[2].user_id)
        self.assertEqual(PassByME2FAClient.MessageStatus.SEEN, session_info.recipient_statuses[2].status)



from setuptools import setup
import sys

if sys.version_info < (2, 7, 9):
    sys.exit('Sorry, Python < 2.7.9 is not supported')

setup(
        name             = "passbyme2fa_client",
        version          = "1.0.7",
        author           = "Microsec ltd.",
        author_email     = "development@passbyme.com",
        description      = "Messaging SDK for PassBy[ME] Mobile ID solution",
        url              = "https://www.passbyme.com/",
        download_url     = "https://github.com/microsec/passbyme2fa-client-python",
        license          = "MIT",
        packages         = ["passbyme2fa_client"],
        package_data     = {"passbyme2fa_client": ["truststore.pem"]},
        tests_require    = [
            "httpretty",
        ],
        install_requires = [
            "arrow",
        ],
        test_suite       = "test",
)



import datetime
import arrow
from .json_helper import get_json_field
from .recipient_status import RecipientStatus

class SessionInfo:

    def __init__(self, json, pbm_client):
        self.re_init(json)
        self.pbm_client = pbm_client

    def refresh(self):
        self.pbm_client.track_message(session_info = self)

    def cancel(self):
        self.pbm_client.cancel_message(session_info = self)

    def re_init(self, json):
        self.message_id = get_json_field(json, "messageId")
        self.expiration_date = arrow.get(
            get_json_field(json, "expirationDate")
        )
        self.recipient_statuses = [
            RecipientStatus(json)
                for json in get_json_field(json, "recipients")
        ]
